#include "sensors.h"

#define ACC_ADDRESS 0x1D
#define MAG_ADDRESS 0x0E
#define GYR_ADDRESS 0x58

const uint8_t ACC_OUT_X_MSB = 0x01;
const uint8_t ACC_F_SETUP= 0x09;
const uint8_t ACC_F_STATUS= 0x00;
const uint8_t ACC_CTRL_REG1= 0x2A;	// Active mode and Data Rate
const uint8_t ACC_CTRL_REG4= 0x2D;	// FIFO Interrupt
const uint8_t ACC_CTRL_REG5= 0x2E;	// Interrupt Pin
const uint8_t ACC_DYN_RANGE_MODE= 0x0E;
const uint8_t ACC_INT_SOURCE= 0x0C;

const uint8_t active_mask= 0x1;		//Active Mode Mask
const uint8_t standby_mask= 0x00;	//Standby Mode Mask
const uint8_t FIFO_mask= 0x40;		//FIFO Interrupt Mask
const uint8_t intpin_mask= 0x40;	//FIFO to Pin 1
const uint8_t fill_mask= 0xA0;		//Fill Buffer

void sensors_init(void)
{
}
uint16_t convert_accData(uint16_t coord_MSB, uint16_t coord_LSB, uint8_t* sign)	//convert Data to gs
{
	uint16_t result=0;
	coord_MSB=coord_MSB<<4;
	coord_LSB=coord_LSB>>4;
	result= coord_MSB|coord_LSB;
	if (result>=0x800)
	{
		result=~result+1;
		result<<=4;
		result>>=4;
		*sign=0;		//result is negative
	}
	else
	{
		*sign=1;		//result is positive
	}
	result =(1000*result+256)>>9;
	return result;
}
static void acc_write2_register(uint8_t reg, uint8_t value)
{
	uint8_t data[2];
	data[0] = reg;
	data[1] = value;

	twi_master_transfer(ACC_ADDRESS<<1, data, 2, TWI_ISSUE_STOP);

}
static void acc_read_register(uint8_t address, int size, uint8_t *data)
{
	twi_master_transfer(MMA8653FC_ADDRESS << 1, &address, 1, TWI_DONT_ISSUE_STOP);
	twi_master_transfer(MMA8653FC_ADDRESS << 1 | TWI_READ_BIT, data, size, TWI_ISSUE_STOP);
}

static void modifiedRead(uint8_t regis, uint8_t* data, uint8_t data_length)
{

	for(int i=0; i<data_length; i++){
		data[i]=0;
	}
	twi_master_transfer(ACC_ADDRESS<<1, &regis, 1, TWI_DONT_ISSUE_STOP);
	twi_master_transfer(ACC_ADDRESS<<1| TWI_READ_BIT, data, data_length, TWI_DONT_ISSUE_STOP);
}

void sensors_start()
{
	uint8_t acc_X_MSB = 0;		//X MSB data
	uint8_t acc_X_LSB = 0;		//X LSB data
	uint8_t acc_Z_LSB = 0;		//Z LSB data etc.
	uint8_t acc_Z_MSB = 0;
	uint8_t acc_Y_LSB = 0;
	uint8_t acc_Y_MSB = 0;

	uint8_t acc_status= 0;		//acc status register
	uint8_t acc_f_setup = 0;	//acc FIFO register
	uint8_t acc_int_source = 0;	//acc interrupt register

	uint16_t acc_X_result = 0;	//resulting acc X data etc.
	uint16_t acc_Y_result = 0;
	uint16_t acc_Z_result = 0;

	uint8_t acc_X_sign=0;		//acc X sign indicator
	uint8_t acc_Y_sign=0;
	uint8_t acc_Z_sign=0;

	if(twi_master_init())
	{
		acc_write2_register(ACC_CTRL_REG1, standby_mask);		//set standby mode with 800 Hz
		acc_write2_register(ACC_F_SETUP, fill_mask);			//set fill buffer FIFO mode
		acc_write2_register(ACC_CTRL_REG4, FIFO_mask);			//Enable FIFO Interrupt
		acc_write2_register(ACC_CTRL_REG5, intpin_mask);		//route FIFO Interrupt to Pin 1
		acc_write2_register(ACC_DYN_RANGE_MODE, active_mask);	//set active mode with 800 Hz
		acc_write2_register(ACC_CTRL_REG1, active_mask);		//set active mode with 800 Hz
	}
	while(1){
		if(twi_master_init())
		{
			acc_read_register(ACC_F_SETUP, 1, &acc_f_setup);		//read FIFO Setup Register to check current FIFO buffer mode and Watermark Level
			acc_read_register(ACC_INT_SOURCE, 1, &acc_int_source);	//read Interrupt Status Register to check Bit 6 for FIFO Interrupt Event (Overflow or Watermark)
			acc_read_register(ACC_F_STATUS, 1, &acc_status);		//read FIFO Status for Buffer Overflow Info and Watermark Sample Count
			modifiedRead(ACC_OUT_X_MSB, &acc_X_MSB, 18);			//store data in acc_X/Y/Z_MSB and acc_X/Y/Z_LSB

			acc_X_result=convert_accData(acc_X_MSB, acc_X_LSB, &acc_X_sign);	//convert data to gs
			acc_Y_result=convert_accData(acc_Y_MSB, acc_Y_LSB, &acc_Y_sign);
			acc_Z_result=convert_accData(acc_Z_MSB, acc_Z_LSB, &acc_Z_sign);

			printf("ACC X: %d", acc_X_result);						//print results
			printf("ACC X Sign: %d", acc_X_sign);
			printf("ACC Y: %d", acc_Y_result);
			printf("ACC Y Sign: %d", acc_Y_sign);
			printf("ACC Z: %d", acc_Z_result);
			printf("ACC Z Sign: %d\n", acc_Z_sign);
			nrf_delay_ms(1250);
		}
	}
}
//}
